// To be licensed -- A telefoninux-community stub.

// Compile with $CC $(pkg-config --cflags --libs elementary) etube-downside.c -o etube-downside

#include <Ecore.h>
#include <Ecore_Con.h>
#include <eina_list.h>
#include <string.h>
#include <stdio.h>

typedef unsigned char bool;
#define false 0
#define true 1

typedef struct _EtubeConData EtubeConData;
struct _EtubeConData {
    char *data;
    int data_size;
};

typedef struct _EtubeHttpConRequest EtubeHttpConRequest;
struct _EtubeHttpConRequest {
    char *host;
    char *type;
    char *headers;
    char *target;
    char *content;
};

typedef struct _EtubeHttpConReply EtubeHttpConReply;
struct _EtubeHttpConReply {
    char *data;
    unsigned int code;
};

typedef struct _EtubeCon EtubeCon;
struct _EtubeCon {
	Ecore_Con_Server *server;
	Ecore_Event_Handler *connection;
	Ecore_Event_Handler *transfer;
	Ecore_Event_Handler *disconnect;
	EtubeHttpConRequest *request;
	EtubeHttpConReply *reply;
	EtubeConData *condata;
};

typedef struct _YTVideo YTVideo;
struct _YTVideo {
	char *id;
	char *title;
	char *author;
	char *thumbnail;
	char *description;
	unsigned int duration;
	unsigned int rating; // data not grabbed yet
	unsigned int raters; // data not grabbed yet
};

void simpleStripHtmlTags(char *str) {
	char *pos, *endpos;
	
	pos = endpos = str;
	while ((pos = strchr(endpos, '<'))) {
		if (endpos = strchr(pos, '>'))
			memmove(pos, endpos+1, strlen(pos) - ((int)endpos - (int)pos));
	}
}

void htmlCharFix(char *str) {
	char *pos, *oldpos;
	unsigned int code, i;
	char scanner[7];
	char charname[5];

	pos = oldpos = str;
	while ((pos = strchr(pos, '&'))) {
		if ((pos+1) != NULL && *(pos+1) == '#') {
			for (i = 3; i > 0; i--) {
				if ((pos+i+2) != NULL && *(pos+i+2) == ';') {
					sprintf(scanner, "&#%%%dd;", i);
					if (sscanf(pos, scanner, &code)) {
						memmove(pos, pos+i+2, ((int)str + strlen(str) - (int)pos));
						*pos = (char) code;
						break;
					}
				}
			}
		} else {
			for (i = 4; i > 0; i--) {
				if ((pos+i+1) != NULL && *(pos+i+1) == ';') {
					sprintf(scanner, "&%%%ds;", i);
					if (sscanf(pos, scanner, &charname)) {
						if (!strcmp(charname, "nbsp"))
							*(pos+i+1) = ' ';
						else if (!strcmp(charname, "quot"))
							*(pos+i+1) = '"';
						else if (!strcmp(charname, "amp"))
							*(pos+i+1) = '&';
						else if (!strcmp(charname, "lt"))
							*(pos+i+1) = '<';
						else if (!strcmp(charname, "gt"))
							*(pos+i+1) = '>';

						if (*(pos+i+1) != ';') {
							memmove(pos, pos+i+1, ((int)str + strlen(str) - (int)pos));
							break;
						}
					}
				}
			}
		}

		if (oldpos == pos)
			pos++;
		oldpos = pos;
	}
}

char *getInnerText(char *src, char *first, char *last, char **fpos, char **lpos) {
	char *dest;
	char *firstpos;
	char *lastpos;
	int srcsize;
	int firstsize;
	int lastsize;
	int destsize;
	
	if (!src)
		return NULL;
	
	srcsize = strlen(src);
	firstsize = strlen(first);
	lastsize = strlen(last);
	
	if (fpos)
		*fpos = NULL;
	if (lpos)
		*lpos = NULL;
	
	if (!srcsize)
		return NULL;
	
	if (firstsize)
		firstpos = strstr(src, first);
	else
		firstpos = src;
	
	if (firstpos) {
		if (lastsize)
			lastpos = strstr(firstpos, last);
		else
			lastpos = src+srcsize;
	}
	
	if (!firstpos || !lastpos)
		return NULL;

	destsize = lastpos - firstpos - firstsize;
	
	dest = (char *) malloc(destsize+1);
	memcpy(dest, firstpos+firstsize, destsize);
	dest[destsize] = '\0';
	
	if (fpos)
		*fpos = firstpos;
	if (lpos)
		*lpos = lastpos;

	return dest;
}

void parse_http_reply(char *reply, EtubeHttpConReply *http_reply) {
	char protocol[4];
	sscanf(reply, "%4s", protocol);
	sscanf(reply+9, "%3d", &http_reply->code);

	http_reply->data = NULL;
	
	if (!strncmp(protocol, "HTTP", 4)) {
		if (http_reply->code == 200) {
			char *http;
			http = strstr(reply, "\r\n\r\n");
			if (http)
				http_reply->data = http + 4;
		}
	}
}

static int etubecon_connect(void *data, int type, void *ev) {
	Ecore_Con_Event_Server_Add *e;
	EtubeConData *reply;
	EtubeHttpConRequest *request;
	
	e = ev;
	request = data;
	reply = ecore_con_server_data_get(e->server);
	
	char httprequest[256];
	
	if (request->type &&
		(!strcasecmp(request->type, "POST") || !strcasecmp(request->type, "GET")) &&
	    request->host && strlen(request->host)) {
		if (!request->target || !strlen(request->target)) {
			request->target = (char *)malloc(2);
			request->target = "/\0";
		}

		sprintf(httprequest, "%s %s HTTP/1.0\r\n"
					 "Host: %s\r\n", request->type, request->target, request->host);
	} else {
		ecore_con_server_del(e->server);
		return false;
	}
	
	if (request->headers && strlen(request->headers))
		sprintf(httprequest, "%s\r\n", httprequest, request->headers);
	
	if (request->content && strlen(request->content) && !strcasecmp(request->type, "POST")) {
		sprintf(httprequest, "%s\r\n"
						 "Content-Type: application/x-www-form-urlencoded\r\n"
						 "Content-length: %d\r\n"
						 "\r\n"
						 "%s\r\n", httprequest, strlen(request->content), request->content);
	}
	
	strcat(httprequest, "\r\n");
	
// 	printf("Connected to server ... \n%s\n",httprequest);

	ecore_con_server_send(e->server, httprequest, strlen(httprequest));
	
	return true;
}

static int etubecon_transfer(void *data, int type, void *ev) {
	Ecore_Con_Event_Server_Data *e;
	EtubeConData *reply;

	e = ev;
	reply = ecore_con_server_data_get(e->server);
	
	reply->data = realloc(reply->data, reply->data_size + e->size + 1);
	memcpy(reply->data + reply->data_size, e->data, e->size);

	reply->data_size += e->size;
	reply->data[reply->data_size] = '\0';
							
	return true;
}

static int etubecon_disconnect(void *data, int type, void *ev) {
	Ecore_Con_Event_Server_Del *e;
	EtubeConData *reply;
	EtubeHttpConReply *http_reply;

	e = ev;
	reply = ecore_con_server_data_get(e->server);
	http_reply = data;
	
	ecore_con_server_del(e->server);
	
	if (reply->data)
		parse_http_reply(reply->data, http_reply);

	free(reply);
	ecore_main_loop_quit();
	return true;
}

bool parseRSSVideoList(char *rss, Eina_List **videolist) {
	YTVideo *video;
	char *item, *tmp;
	
	*videolist = NULL;
	
	if (!rss || !strlen(rss))
		return false;

	tmp = (char *) malloc(strlen(rss));
	strcpy(tmp, rss);
// 	tmp = rss; - using this it would free the original pointer

	while ((item = getInnerText(tmp, "<item>", "</item>", NULL, &tmp))) {
		video = (YTVideo*) malloc(sizeof(YTVideo));
	
		video->id = getInnerText(item, "<yt:videoid>", "</yt:videoid>", NULL, NULL);
		video->title = getInnerText(item, "<title>", "</title>", NULL, NULL);
		video->author = getInnerText(item, "<author>", "</author>", NULL, NULL);
		video->description = getInnerText(item, "<media:description type='plain'>",
										"</media:description>", NULL, NULL);
		video->duration = atoi(getInnerText(item, "<yt:duration seconds='", "\'/>", NULL, NULL));
		//video->thumbnail = //TODO
		
		htmlCharFix(video->title);
		htmlCharFix(video->author);
		htmlCharFix(video->description);
		simpleStripHtmlTags(video->description);

		*videolist = eina_list_append(*videolist, video);
		
		free(item); // For sure!
	}

	free(tmp);
	
	if (*videolist)
		return true;
	else
		return false;
}

void etubecon_init(EtubeCon *con) {
	printf("DEBUG: <etubecon_init>\n");
	con->connection = NULL;
	con->transfer = NULL;
	con->disconnect = NULL;
	con->request = calloc(1, sizeof(EtubeHttpConRequest));
	con->condata = calloc(1, sizeof(EtubeConData));
	con->reply = calloc(1, sizeof(EtubeHttpConReply));
	printf("DEBUG: </etubecon_init>\n");
}

void etubecon_shutdown(EtubeCon *con) {
	ecore_event_handler_del(con->connection);
	ecore_event_handler_del(con->disconnect);
	ecore_event_handler_del(con->transfer);
	free(con->request);
	free(con->reply);
// 	free(con);
}

void etubecon_httpget(EtubeCon *con) {
	con->connection = ecore_event_handler_add(ECORE_CON_EVENT_SERVER_ADD, etubecon_connect, con->request);
	con->transfer = ecore_event_handler_add(ECORE_CON_EVENT_SERVER_DATA, etubecon_transfer, NULL);
	con->disconnect = ecore_event_handler_add(ECORE_CON_EVENT_SERVER_DEL, etubecon_disconnect, con->reply);

	ecore_con_server_connect(ECORE_CON_REMOTE_SYSTEM, con->request->host, 80, con->condata);
}

void etube_search_videos(EtubeCon *econ, char *query) {

   printf("DEBUG: <etube_search_videos>\n");
	if (econ->connection) {
		etubecon_shutdown(econ);
		etubecon_init(econ);
	}
	
	econ->request->type = "GET";
	econ->request->host = "gdata.youtube.com";
	econ->request->target = "/feeds/api/videos?v=2&alt=rss&q=";
	if (query && strlen(query)) {
		char *tmp;
		tmp = (char *) malloc(strlen(econ->request->target) + strlen(query) + 1);
		strcpy(tmp, econ->request->target);
		strcat(tmp, query);
		econ->request->target = tmp;
	}
	econ->request->headers = "User-Agent: eYT/0.1";

   printf("DEBUG:    <etubecon_httpget>\n");
	etubecon_httpget(econ);
   printf("DEBUG:    </etubecon_httpget>\n");

   printf("DEBUG: </etube_search_videos>\n");
}

void etube_get_video_page(EtubeCon *econ, char *id) {
	if (econ->connection) {
		etubecon_shutdown(econ);
		etubecon_init(econ);
	}
	
	econ->request->type = "GET";
	econ->request->host = "www.youtube.com";
	econ->request->target = "/watch?v=tQPjfUqp-dk";
// 	if (id && strlen(id)) {
// 		char *tmp; // FIXME segfault
// 		tmp = (char *) malloc(strlen(econ->request->target) + strlen(id) + 1);
// 		strcpy(tmp, econ->request->target);
// 		strcat(tmp, id);
// 		econ->request->target = tmp;
// 	}
	econ->request->headers = "User-Agent: eYT/0.1";
	etubecon_httpget(econ);
}

void etube_print_video_titles(Eina_List *videolist) {
	YTVideo *vid;
	Eina_List *tlist;
	
	EINA_LIST_FOREACH(videolist, tlist, vid) {
		printf("%s\n", vid->title);
	}
	free(tlist);
}

int downside_main(int argc, char **argv) {
	EtubeCon econ;

	Eina_List *videolist;
	
	etubecon_init(&econ);

	ecore_init();
	ecore_con_init();
	eina_list_init();
	
	etube_search_videos(&econ, "openmoko");

	ecore_main_loop_begin();

	printf("Http code: %d\n", econ.reply->code);
	parseRSSVideoList(econ.reply->data, &videolist);
	if (econ.reply)
		etube_print_video_titles(videolist);


	etube_search_videos(&econ, NULL);
	
	ecore_main_loop_begin();
	
	parseRSSVideoList(econ.reply->data, &videolist);
	if (econ.reply)
		etube_print_video_titles(videolist);
		
	
	etube_get_video_page(&econ, "tQPjfUqp-dk");
	ecore_main_loop_begin();
	if (econ.reply)
		printf("Secret 't': %s\n", getInnerText(econ.reply->data, "\"t\": \"", "\",", NULL, NULL));
		
	ecore_con_shutdown();
	ecore_shutdown();
	eina_list_shutdown();
}

