/*
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; version 2 of the
    License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA  02110-1301, USA.

    ---
    Copyright (C) 2009, Marco Trevisan (Treviño) <mail@3v1n0.net>
    Copyright (C) 2009, Igor Scabini <furester@gmail.com>

    It can be compiled with:
    $CC etube-test.c $(pkg-config --cflags --libs elementary) -o etube-test
 */

#include <Elementary.h>
#include <Ecore_Con.h>

#define ETUBE_VERSION "0.3"
#define TMP_PATH "/tmp/.etube"
#define DEBUG 1

typedef unsigned char bool;
#define false 0
#define true 1

typedef struct _EtubeConData EtubeConData;
struct _EtubeConData {
	char *data;
	int data_size;
};

typedef struct _EtubeHttpConRequest EtubeHttpConRequest;
struct _EtubeHttpConRequest {
	char *host;
	char *type;
	char *headers;
	char *target;
	char *content;
	bool connected;
};

typedef struct _EtubeHttpConReply EtubeHttpConReply;
struct _EtubeHttpConReply {
	char *data;
	unsigned int code;
};

typedef struct _EtubeCon EtubeCon;
struct _EtubeCon {
	Ecore_Con_Server *server;
	Ecore_Event_Handler *connection;
	Ecore_Event_Handler *transfer;
	Ecore_Event_Handler *disconnect;
	EtubeHttpConRequest *request;
	EtubeHttpConReply *reply;
	EtubeConData *condata;
};

//TODO - move to eina_string_share_add/del and to const char*;
typedef struct _YTVideo YTVideo;
struct _YTVideo {
	char *id;
	char *title;
	char *author;
	char *duration;
	char *thumbnail;
	char *description;
	unsigned int seconds;
	unsigned int rating; // data not grabbed yet
	unsigned int raters; // data not grabbed yet
};

typedef struct _EtubeWindow EtubeWindow;
struct _EtubeWindow {
	Evas_Object *window;
	Evas_Object *frametitle;
	Evas_Object *scrollist;
	Evas_Object *searchbox;
	Evas_Object *searchentry;
	Evas_Object *searchscroll;
	Evas_Object *buttonsbox;
	Evas_Object *lbutton;
	Evas_Object *rbutton;
	Evas_Object *inwin;
	Evas_Object *inwintext;
	Evas_Object *etubepager;
	Evas_Object *videospage;
};

typedef struct _EtubeVideoPage EtubeVideoPage;
struct _EtubeVideoPage {
	YTVideo *video;
	EtubeWindow *etwin;
	Elm_List_Item *vitem;
};

typedef struct _EtubeSearch EtubeSearch;
struct _EtubeSearch {
	bool standard;
	char *query;
};

typedef struct _EtubeStruct EtubeStruct;
struct _EtubeStruct {
	YTVideo *video;
	EtubeCon *etcon;
	Eina_List *videolist;
	EtubeWindow *etwin;
	EtubeSearch *search;
};

void simpleStripHtmlTags(char *str) {
	char *pos, *endpos;
	
	pos = endpos = str;
	while ((pos = strchr(endpos, '<'))) {
		if (endpos = strchr(pos, '>'))
			memmove(pos, endpos+1, strlen(pos) - ((int)endpos - (int)pos));
	}
}

void carriageRemove(char *str) {
	char *pos;

	pos = str;
	while ((pos = strchr(pos, '\r'))) {
		memmove(pos, pos+1, strlen(pos));
	}
}

void trimString(char *str) {
	char *pos;

	pos = str;
	while (pos && *pos != '\0' && isspace(*pos)) {
		pos++;
	}
	
	if (pos != str)
		memmove(str, pos, strlen(pos)+1);
	
	if (!strlen(str))
		return;

	pos = str + strlen(str) - 1;
	while (pos && isspace(*pos)) {
		*pos = '\0';
		pos--;
	}
}

void htmlCharFix(char *str) {
	char *pos, *oldpos;
	unsigned int code, i, newstrlen;
	char scanner[7];
	char charname[5];

	if (str)
		newstrlen = strlen(str);

	pos = oldpos = str;
	while ((pos = strchr(pos, '&'))) {
		if ((pos+1) != NULL && *(pos+1) == '#') {
			for (i = 3; i > 0; i--) {
				if ((pos+i+2) != NULL && *(pos+i+2) == ';') {
					sprintf(scanner, "&#%%%dd;", i);
					if (sscanf(pos, scanner, &code)) {
						newstrlen -= i-2;
						memmove(pos, pos+i+2, ((int)str + newstrlen - ((int)pos+2)));
						*pos = (char) code;
						break;
					}
				}
			}
		} else {
			for (i = 4; i > 0; i--) {
				if ((pos+i+1) != NULL && *(pos+i+1) == ';') {
					sprintf(scanner, "&%%%ds;", i);
					if (sscanf(pos, scanner, &charname)) {
						if (!strcmp(charname, "nbsp"))
							*(pos+i+1) = ' ';
						else if (!strcmp(charname, "quot"))
							*(pos+i+1) = '"';
						else if (!strcmp(charname, "amp"))
							*(pos+i+1) = '&';
						else if (!strcmp(charname, "lt"))
							*(pos+i+1) = '<';
						else if (!strcmp(charname, "gt"))
							*(pos+i+1) = '>';

						if (*(pos+i+1) != ';') {
							newstrlen -= i-1;
							memmove(pos, pos+i+1, ((int)str + newstrlen - ((int)pos+1)));
							break;
						}
					}
				}
			}
		}

		if (oldpos == pos)
			pos++;
		oldpos = pos;
	}
}

char *getInnerText(char *src, char *first, char *last, char **fpos, char **lpos) {
	char *dest;
	char *firstpos;
	char *lastpos;
	int srcsize;
	int firstsize;
	int lastsize;
	int destsize;
	
	if (!src)
		return NULL;
	
	srcsize = strlen(src);
	
	if (first)
		firstsize = strlen(first);
	else
		firstsize = false;
		
	if (last)
		lastsize = strlen(last);
	else
		lastsize = false;
	
	if (fpos)
		*fpos = NULL;
	if (lpos)
		*lpos = NULL;
	
	if (!srcsize)
		return NULL;
	
	if (firstsize)
		firstpos = strstr(src, first);
	else
		firstpos = src;
	
	if (firstpos) {
		if (lastsize)
			lastpos = strstr(firstpos, last);
		else
			lastpos = src+srcsize;
	}
	
	if (!firstpos || !lastpos)
		return NULL;

	destsize = lastpos - firstpos - firstsize;
	
	dest = (char *) malloc((destsize+1)*sizeof(char));
	memcpy(dest, firstpos+firstsize, destsize);
	dest[destsize] = '\0';
	
	if (fpos)
		*fpos = firstpos;
	if (lpos)
		*lpos = lastpos;

	return dest;
}

char *secondsToDurationString(unsigned int seconds) {
	int hours = 0;
	int minutes = 0;
	char *str;
	
	minutes = seconds/60;
	seconds %= 60;
	if (minutes > 60) {
		hours = minutes/60;
		minutes %= 60;
	}
	
	str = (char *) malloc(9*sizeof(char));
	str[0] = '\0';
	if (hours) {
		if (hours < 10)
			sprintf(str, "0%d:", hours);
		else
			sprintf(str, "%d:", hours);
	}
	
	if (minutes < 10)
		sprintf(str, "%s0%d:", str, minutes);
	else
		sprintf(str, "%s%d:", str, minutes);
	
	if (seconds < 10)
		sprintf(str, "%s0%d", str, seconds);
	else
		sprintf(str, "%s%d", str, seconds);

	return str;
}

char *smartNewLine(char *str, int maxlenght) {
	char *out, *spacepos, *tmp;
	
	if (!str)
		return NULL;
	
	if (!strlen(str))
		return NULL;
	
	out = malloc((strlen(str)+1)*sizeof(char));
	strcpy(out, str);
	spacepos = out;
	tmp = out;
	
	while ((tmp = strchr(tmp, '\n'))) {
		*tmp = ' ';
		tmp++;
	}

	tmp = out;
	while ((spacepos = strchr(spacepos, ' '))) {
		if (((int)spacepos - (int)tmp) > maxlenght) {
			*spacepos = '\n';
			tmp = spacepos;
		} else
			spacepos++;
	}

	return out;
}

char *urlEscape(char *str) {
	unsigned int i, newsize;
	char *tmp;
	char escaped[4];
	char invalid[] = {' ', '.', '\'', '-', '!', '?', '+', '&', '*', '/',
				   '(', ')', '@', '$', ':', '%', ';', '=', '_', '|'};
	
	if (str)
		newsize = strlen(str);
	else
		return NULL;
	
	tmp = str;
	while (tmp && *tmp != '\0') {
		for (i = 0; i < sizeof(invalid); i++) {
			if (*tmp == invalid[i]) {
				newsize += 2;
				break;
			}
		}
		tmp++;
	}
	
	
	if (newsize > strlen(str)) {
		tmp = malloc((newsize+1)*sizeof(char));
		strcpy(tmp, str);
		str = tmp;
	
		tmp = str;
		while (tmp && *tmp != '\0') {
			for (i = 0; i < sizeof(invalid); i++) {
				if (*tmp == invalid[i]) {
					memmove(tmp+2, tmp, (int)str + newsize - (int)tmp - 1);
					sprintf(escaped, "%%%X", (int)invalid[i]);
					memcpy(tmp, escaped, 3);
					break;
				}
			}
			tmp++;
		}
	}
	
	return str;
}

void parse_http_reply(char *reply, EtubeHttpConReply *http_reply) {
	char protocol[5];
	sscanf(reply, "%4s", protocol);
	sscanf(reply+9, "%3d", &http_reply->code);

	http_reply->data = NULL;
	
	if (!strncmp(protocol, "HTTP", 4)) {
		if (http_reply->code == 200) { // Malformed request is 401
			char *content;
			content = getInnerText(reply, "\r\n\r\n", NULL, NULL, NULL);
			if (content) {
				http_reply->data = content;
			}
		} else if (http_reply->code == 301 || http_reply->code == 302) {
			char *location;
			location = getInnerText(reply, "Location: ", "\r\n", NULL, NULL);
			if (location) {
				http_reply->data = location;
			}
		}
	}
}

static int etubecon_connect(void *data, int type, void *ev) {
	Ecore_Con_Event_Server_Add *e;
	EtubeConData *reply;
	EtubeHttpConRequest *request;
	char httprequest[512];
	
	e = ev;
	request = data;
	reply = ecore_con_server_data_get(e->server);
	
	if (request->type &&
		(!strcasecmp(request->type, "POST") || !strcasecmp(request->type, "GET") ||
		 !strcasecmp(request->type, "HEAD")) && request->host && strlen(request->host)) {
		if (!request->target || !strlen(request->target)) {
			request->target = (char *)malloc(2*sizeof(char));
			request->target = "/\0";
		}

		sprintf(httprequest, "%s %s HTTP/1.0\r\n"
					 "Host: %s\r\n", request->type, request->target, request->host);
	} else {
		ecore_con_server_del(e->server);
		return false;
	}
	
	if (request->headers && strlen(request->headers))
		sprintf(httprequest, "%s\r\n", httprequest, request->headers);
	
	if (request->content && strlen(request->content) && !strcasecmp(request->type, "POST")) {
		sprintf(httprequest, "%s\r\n"
						 "Content-Type: application/x-www-form-urlencoded\r\n"
						 "Content-length: %d\r\n"
						 "\r\n"
						 "%s\r\n", httprequest, strlen(request->content), request->content);
	}
	
	strcat(httprequest, "\r\n");
	
#ifdef DEBUG
	printf("Connected to server; Sending request... \n%s\n",httprequest);
#endif

	request->connected = true;
	ecore_con_server_send(e->server, httprequest, strlen(httprequest));

	return true;
}

static int etubecon_transfer(void *data, int type, void *ev) {
	Ecore_Con_Event_Server_Data *e;
	EtubeConData *reply;

	e = ev;
	reply = ecore_con_server_data_get(e->server);
	
	reply->data = realloc(reply->data, reply->data_size + e->size + sizeof(char));
	memcpy(reply->data + reply->data_size, e->data, e->size);

	reply->data_size += e->size;
	reply->data[reply->data_size] = '\0';
	
	return true;
}

static int etubecon_disconnect(void *data, int type, void *ev) {
	Ecore_Con_Event_Server_Del *e;
	EtubeConData *reply;
	EtubeHttpConReply *http_reply;

	e = ev;
	reply = ecore_con_server_data_get(e->server);
	http_reply = data;
	
	ecore_con_server_del(e->server);
	
	if (reply && reply->data)
		parse_http_reply(reply->data, http_reply);

	free(reply->data);
	free(reply);
	return true;
}

bool parseRSSVideoList(char *rss, Eina_List **videolist) {
	YTVideo *video;
	char *item, *tmp;
	
	*videolist = NULL;
	
	if (!rss || !strlen(rss))
		return false;

	tmp = (char *) malloc((strlen(rss)+1)*sizeof(char));
	strcpy(tmp, rss);

	while ((item = getInnerText(tmp, "<item>", "</item>", NULL, &tmp))) {
		video = (YTVideo*) malloc(sizeof(YTVideo));
	
		video->id = getInnerText(item, "<yt:videoid>", "</yt:videoid>", NULL, NULL);
		video->title = getInnerText(item, "<title>", "</title>", NULL, NULL);
		video->author = getInnerText(item, "<author>", "</author>", NULL, NULL);
		video->description = getInnerText(item, "<media:description type='plain'>",
										"</media:description>", NULL, NULL);
		video->duration = getInnerText(item, "<yt:duration seconds='", "\'/>", NULL, NULL);
		video->seconds = atoi(video->duration);
		free(video->duration);
		video->duration = secondsToDurationString(video->seconds);
		//video->thumbnail = //TODO
		
		htmlCharFix(video->title);
		htmlCharFix(video->author);
		htmlCharFix(video->description);
		simpleStripHtmlTags(video->description);
		carriageRemove(video->description);

		*videolist = eina_list_append(*videolist, video);

		free(item);
	}

	free(tmp);
	
	if (*videolist)
		return true;
	else
		return false;
}

void etubecon_init(EtubeStruct *et) {
	ecore_con_init();
	EtubeCon *con = calloc(1, sizeof(EtubeCon));
	con->server = NULL;
	con->connection = NULL;
	con->transfer = NULL;
	con->disconnect = NULL;
	con->request = calloc(1, sizeof(EtubeHttpConRequest));
	con->request->connected = false;
	con->condata = calloc(1, sizeof(EtubeConData));
	con->condata->data_size = 0;
	con->reply = calloc(1, sizeof(EtubeHttpConReply));
	et->etcon = con;
}

void etubecon_shutdown(EtubeStruct *et) {
	if (et->etcon) {
		ecore_event_handler_del(et->etcon->connection);
		ecore_event_handler_del(et->etcon->disconnect);
		ecore_event_handler_del(et->etcon->transfer);
		ecore_con_server_del(et->etcon->server); // here?
		ecore_con_shutdown();
		free(et->etcon->reply->data);
		free(et->etcon->reply);
// 		free(et->etcon->condata->data);
// 		free(et->etcon->condata);
// 		free(et->etcon->request->headers);
// 		free(et->etcon->request->content);
// 		free(et->etcon->request->target);
// 		free(et->etcon->request->type);
		free(et->etcon->request);
		free(et->etcon);
		et->etcon = NULL;
	}
}

//FIXME crashes after some retries!
int etubecon_httpget_retry(void *data) {
	static unsigned int retries = 1;
	EtubeCon *con;
	con = data;

// 	printf("Connection (%x) retry %d...\n",con->server, retries);
	//Define MAX_RETRIES
	if (retries < 10 && con && con->request && !con->request->connected) { //FIXME check that con != NULL
		if (con->server) {
			ecore_con_server_del(con->server);
			con->server = NULL;
		}
	
		con->server = ecore_con_server_connect(ECORE_CON_REMOTE_SYSTEM,
									    con->request->host, 80, con->condata);
		retries++;
		return true;
	}
	
	retries = 1;
	return false;
}

void etubecon_httpget(EtubeCon *con) {
	if (!con->connection)
		con->connection = ecore_event_handler_add(ECORE_CON_EVENT_SERVER_ADD,
										  etubecon_connect, con->request);
	if (!con->transfer)
		con->transfer = ecore_event_handler_add(ECORE_CON_EVENT_SERVER_DATA,
										  etubecon_transfer, NULL);
	if (!con->disconnect)
		con->disconnect = ecore_event_handler_add(ECORE_CON_EVENT_SERVER_DEL,
										  etubecon_disconnect, con->reply);

	if (con->server) {
		ecore_con_server_del(con->server);
		con->server = NULL;
	}
	con->server = ecore_con_server_connect(ECORE_CON_REMOTE_SYSTEM, // Segfault!?
								    con->request->host, 80, con->condata);
	//ecore_timer_add(2.0, etubecon_httpget_retry, con); //FIXME
}

void etube_search_videos(EtubeCon *econ, char *query) {
	if (econ->connection) {
		return; // we need to avoid multiple requests!
// 		etubecon_shutdown(econ); // set to et!
// 		etubecon_init(econ); // set to et!
	}
	
	econ->request->type = "GET";
	econ->request->host = "gdata.youtube.com";
	econ->request->target = "/feeds/api/videos?v=2&alt=rss&q="; //use mobile?
	if (query && strlen(query)) {
		char *tmp;
		tmp = (char *) malloc((strlen(econ->request->target) + strlen(query) + 1)*sizeof(char));
		strcpy(tmp, econ->request->target);
		strcat(tmp, query);
		econ->request->target = tmp;
	}
	econ->request->headers = "User-Agent: eYT/0.1";
	etubecon_httpget(econ);
}

void etube_search_feed(EtubeCon *econ, char *feed, char *time) {
	char str[1024];
	
	if (econ->connection) {
		return; // we need to avoid multiple requests!
// 		etubecon_shutdown(econ); // set to et!
// 		etubecon_init(econ); // set to et!
	}
	
	econ->request->type = "GET";
	econ->request->host = "gdata.youtube.com";

	if (feed && strlen(feed)) {
		char *tmp;
		sprintf(str, "/feeds/api/standardfeeds/%s", feed);
		
		if (time && strlen(time))
			sprintf(str, "%s?v=2&alt=rss&time=%s", str, time);

		tmp = malloc(strlen(str)+1);
		strcpy(tmp, str);
		econ->request->target = tmp;
	} else {
		econ->request->target = "/feeds/api/standardfeeds/top_rated?v=2&alt=rss";
	}

	econ->request->headers = "User-Agent: eYT/0.1";
	etubecon_httpget(econ);
}

void etube_get_video_page(EtubeCon *econ, char *id) {
	if (econ->connection) {
// 		etubecon_shutdown(econ);
// 		etubecon_init(econ);
	}
	
	econ->request->type = "GET";
	econ->request->host = "www.youtube.com";
	econ->request->target = "/watch?v=tQPjfUqp-dk";
// 	if (id && strlen(id)) {
// 		char *tmp; // FIXME segfault
// 		tmp = (char *) malloc(strlen(econ->request->target) + strlen(id) + 1);
// 		strcpy(tmp, econ->request->target);
// 		strcat(tmp, id);
// 		econ->request->target = tmp;
// 	}
	econ->request->headers = "User-Agent: eYT/0.1";
	etubecon_httpget(econ);
}

void etube_print_video_titles(Eina_List *videolist) {
	YTVideo *vid;
	Eina_List *tlist;
	
	EINA_LIST_FOREACH(videolist, tlist, vid) {
		printf("%s\n", vid->title);
	}
	free(tlist);
}

//////////////////////////////////
// GUI CODE
//////////////////////////////////

// this fun adds a edje layout,
// to be used asap
void window_swallow(Evas_Object * ly, const char *key, Evas_Object *object) {
	if (ly != NULL)
		edje_object_part_swallow(elm_layout_edje_get(ly),key,object);
}

// close win and free every elm part
static void win_del(void *data, Evas_Object *obj, void *event_info) {
	printf("DEBUG: <win_del>\n");
	elm_exit();
	// TODO add the etube_shutdown!
	printf("DEBUG: </win_del>\n");
}

static int etube_inwin_hide(void *data) {
	EtubeWindow *etwin = data;
	evas_object_hide(etwin->inwin);

	return false;
}

static void etube_video_view_cb(void *data, Evas_Object *obj, void *event_info) {
	EtubeVideoPage *vpage;
	char cmd[120];

	vpage = data;
	
	elm_entry_entry_set(vpage->etwin->inwintext, "<em>Loading mplayer...</><br>"
										"Please, wait!");
	evas_object_show(vpage->etwin->inwin);
	ecore_timer_add(15.0, etube_inwin_hide, vpage->etwin);

	sprintf(cmd, "mplayer -fs -dr -vo xv -vf framestep=2 "
			   "\"http://3v1n0.net/youtube-full-dl/index.php?download=flv&ytv=%s\"", vpage->video->id);
	printf("%s\n",cmd);
	ecore_exe_run(cmd, NULL);
}

static void etube_videolist_pager_view(void *data, Evas_Object *obj, void *event_info) {
	EtubeVideoPage *vpage;
	vpage = data;

	elm_pager_content_promote(vpage->etwin->etubepager, vpage->etwin->videospage);
	evas_object_resize(vpage->etwin->window, 480, 640);
}

static void etube_videolist_select_cb(void *data, Evas_Object *obj, void *event_info) {
	Evas_Object *bx, *bx2, *sc, *ab, *ic, *bt, *fr, *lb, *en;
	EtubeVideoPage *vpage;
	YTVideo *video;
	char *content;

	vpage = data;
	video = vpage->video;

#ifdef DEBUG
	char *descnl = smartNewLine(video->description, 50);
	printf("\nID: %s\n"
		  "Title: %s\n"
		  "Author: %s\n"
		  "Duration: %s\n"
		  "Description: %s\n", video->id, video->title, video->author,
						   video->duration, descnl);
	free(descnl);
#endif

	bx = elm_box_add(vpage->etwin->window);
	elm_win_resize_object_add(vpage->etwin->window, bx);
	evas_object_show(bx);
	
	bx2 = elm_box_add(vpage->etwin->window);
	elm_box_horizontal_set(bx2, true);
	elm_box_homogenous_set(bx2, false);
	evas_object_size_hint_weight_set(bx2, 1.0, 0.0);
	evas_object_size_hint_align_set(bx2, -1.0, -1.0);;
	
	fr = elm_frame_add(vpage->etwin->window);
	elm_frame_style_set(fr, "outdent_top");
	evas_object_size_hint_weight_set(fr, 0.0, 0.0);
	evas_object_size_hint_align_set(fr, 0.5, 0.5);
	elm_box_pack_end(bx, fr);
	evas_object_show(fr);
	
	asprintf(&content, "<b>%s</>", elm_entry_utf8_to_markup(smartNewLine(video->title, 25)));
	en = elm_entry_add(vpage->etwin->window);
	elm_entry_single_line_set(en, FALSE);
	elm_entry_editable_set(en, FALSE);
	elm_entry_line_wrap_set(en, FALSE);
	elm_entry_entry_set(en, content);
	evas_object_size_hint_weight_set(en, 1.0, 0.0);
	evas_object_size_hint_align_set(en, 0.5, 0.5);
	elm_frame_content_set(fr, en);
	evas_object_show(en);
	
	sc = elm_scroller_add(vpage->etwin->window);
	evas_object_size_hint_weight_set(sc, 1.0, 1.0);
	evas_object_size_hint_align_set(sc, -1.0, -1.0);
	elm_scroller_content_min_limit(sc, 0, 0);
	elm_box_pack_end(bx, sc);
	evas_object_show(sc);
	
	bx2 = elm_box_add(vpage->etwin->window);
	evas_object_size_hint_weight_set(bx2, 1.0, 0.0);
	evas_object_size_hint_align_set(bx2, -1.0, -1.0);
	elm_scroller_content_set(sc, bx2);
	evas_object_show(bx2);
	
	//sprintf(content, "<b>%s</>", video->title);
	//ab = elm_anchorblock_add(vpage->etwin->window);
	//elm_anchorblock_text_set(ab, content);
	//evas_object_size_hint_align_set(ab, -1.0, 0.0);
	//evas_object_size_hint_align_set(ab, -1.0, -1.0);
	//elm_box_pack_end(bx2, ab);
	//evas_object_show(ab);
	
	ic = elm_icon_add(vpage->etwin->window);

	if (ecore_file_can_read(video->thumbnail))
		elm_icon_file_set(ic, video->thumbnail, NULL);
	else
		elm_icon_standard_set(ic, "clock");

	elm_icon_scale_set(ic, false, false);
	elm_object_scale_set(ic, 1.5);
	elm_icon_fill_outside_set(ic, false);
	elm_box_pack_end(bx2, ic);
	evas_object_show(ic);
	
	asprintf(&content, "Author: %s<br>"
				    "Duration: %s<br>"
				    "<br>"
				    "<em>%s</>"
				    "<br>", video->author, video->duration,
				    elm_entry_utf8_to_markup(video->description));
	
	ab = elm_anchorblock_add(vpage->etwin->window);
	elm_anchorblock_text_set(ab, content);
	evas_object_size_hint_align_set(ab, -1.0, 0.0);
	evas_object_size_hint_align_set(ab, -1.0, -1.0);
	elm_box_pack_end(bx2, ab);
	evas_object_show(ab);
	
	bx2 = elm_box_add(vpage->etwin->window);
	elm_box_horizontal_set(bx2, 1);
	elm_box_homogenous_set(bx2, 1);
	evas_object_size_hint_weight_set(bx2, 1.0, 0.0);
	evas_object_size_hint_align_set(bx2, -1.0, -1.0);

	bt = elm_button_add(vpage->etwin->window);
	elm_button_label_set(bt, "Back");
	evas_object_smart_callback_add(bt, "clicked", etube_videolist_pager_view, vpage);
	evas_object_size_hint_align_set(bt, -1.0, -1.0);
	evas_object_size_hint_weight_set(bt, 1.0, 0.0);
	elm_box_pack_end(bx2, bt);
	evas_object_show(bt);
	
	bt = elm_button_add(vpage->etwin->window);
	elm_button_label_set(bt, "View");
	evas_object_smart_callback_add(bt, "clicked", etube_video_view_cb, vpage);
	evas_object_size_hint_align_set(bt, -1.0, -1.0);
	evas_object_size_hint_weight_set(bt, 1.0, 0.0);
	elm_box_pack_end(bx2, bt);
	evas_object_show(bt);
	
	bt = elm_button_add(vpage->etwin->window);
	elm_button_label_set(bt, "Download");
	evas_object_smart_callback_add(bt, "clicked", NULL, NULL);
	evas_object_size_hint_align_set(bt, -1.0, -1.0);
	evas_object_size_hint_weight_set(bt, 1.0, 0.0);
	elm_box_pack_end(bx2, bt);
	evas_object_show(bt);
	
	elm_box_pack_end(bx, bx2);
	evas_object_show(bx2);
	elm_pager_content_push(vpage->etwin->etubepager, bx);
	
	evas_object_resize(vpage->etwin->window, 480, 640);
	
	free(content);
}

void etube_videolist_free(EtubeStruct *et) {
	Eina_List *tmp;
	YTVideo *ytv;

	if (et->videolist) {
		EINA_LIST_FOREACH(et->videolist, tmp, ytv) {
			free(ytv->id);
			free(ytv->title);
			free(ytv->author);
			free(ytv->duration);
			free(ytv->thumbnail);
			free(ytv->description);
			free(ytv);
		}
		free(tmp);
	}
	
	//EINA_LIST_FREE(et->videolist, ytv)
	//     free(ytv->id);
}

void etube_videolist_pic_update(void *data, const char *file, int status) {
	EtubeVideoPage *vpage;
	Evas_Object *ic;
	
	vpage = data;

	if (vpage->vitem) {
		ic = elm_icon_add(vpage->etwin->window);
		elm_icon_file_set(ic, file, NULL);
		elm_icon_scale_set(ic, true, false);
		elm_object_scale_set(ic, 0.4);
		elm_list_item_insert_before(vpage->etwin->scrollist, vpage->vitem, vpage->video->title,
							   ic, NULL, etube_videolist_select_cb, vpage);
		elm_list_item_del(vpage->vitem);
		elm_list_go(vpage->etwin->scrollist);
	}
}

void etube_videolist_populate(EtubeStruct *et) {
	YTVideo *vid;
	Eina_List *tmp;
	Evas_Object *ic;
	EtubeVideoPage *vpage;
	Elm_List_Item *vitem;
	char picurl[55];

	while ((tmp = (Eina_List *)elm_list_items_get(et->etwin->scrollist))) {
		vpage = (EtubeVideoPage *)elm_list_item_data_get(tmp->data);
		if (vpage)
			free(vpage);
		elm_list_item_del(tmp->data);
		//ecore_file_unlink( ) here?
	}

	EINA_LIST_FOREACH(et->videolist, tmp, vid) {
#ifdef DEBUG
		printf("%s (%s)\n",vid->title, vid->duration);
#endif
		vpage = calloc(1, sizeof(EtubeVideoPage));
		vpage->etwin = et->etwin;
		vpage->video = vid;
		vid->thumbnail = (char *) malloc((strlen(TMP_PATH)+strlen(vid->id)+6)*sizeof(char));
		sprintf(vid->thumbnail, TMP_PATH"/%s.jpg", vid->id);
		ic = elm_icon_add(et->etwin->window);
		elm_icon_scale_set(ic, true, false);

		if (ecore_file_can_read(vid->thumbnail)) {
			elm_icon_file_set(ic, vid->thumbnail, NULL);
			elm_object_scale_set(ic, 0.4);
		} else {
			elm_icon_standard_set(ic, "clock");
			sprintf(picurl, "http://img.youtube.com/vi/%s/default.jpg", vid->id); //hqdefault?
		}

		vpage->vitem = elm_list_item_append(et->etwin->scrollist, vid->title, ic, NULL,
									 etube_videolist_select_cb, vpage);
		
		if (!ecore_file_can_read(vid->thumbnail))
			ecore_file_download(picurl, vid->thumbnail, etube_videolist_pic_update, NULL, vpage);
	}
	
	evas_object_hide(et->etwin->inwin);
	elm_list_go(et->etwin->scrollist);
	elm_pager_content_promote(et->etwin->etubepager, et->etwin->videospage);

	if (tmp)
		free(tmp);
}

int etube_videolist_check(void *data) {
	EtubeStruct *et;
	et = data;

	if (et->etcon && et->etcon->reply) {
		if (et->etcon->reply->code == 200 && et->etcon->reply->data) {
			parseRSSVideoList(et->etcon->reply->data, &et->videolist);
			//etube_print_video_titles(et->videolist);
			etube_videolist_populate(et);
	
			etubecon_shutdown(et); //Check if this is ok for new searches
			return false;
		} else if (et->etcon->reply->code == 301 || et->etcon->reply->code == 302) {
			printf("Redirecting to %s...\n", et->etcon->reply->data);
			// TODO write redirecting code here
		}
	}
	return true;
}

static void etube_frame_title_set(EtubeStruct *et) {
	if (et->search) {
		if (et->search->query) {
			if (strlen(et->search->query)) {
				char str[1024];
				
				if (et->search->standard) {
					char search[80];
					search[0] = '\0';
			
					if (!strcmp(et->search->query, "top_rated")) {
						strcpy(search, "Top Rated");
					} else if (!strcmp(et->search->query, "top_favorites")) {
						strcpy(search, "Top Favorites");
					} else if (!strcmp(et->search->query, "most_viewed")) {
						strcpy(search, "Top Viewed");
					} else if (!strcmp(et->search->query, "most_discussed")) {
						strcpy(search, "Most Discussed");
					} else if (!strcmp(et->search->query, "most_popular")) {
						strcpy(search, "Most Popular");
					} else if (!strcmp(et->search->query, "most_linked")) {
						strcpy(search, "Most Linked");
					} else if (!strcmp(et->search->query, "most_responded")) {
						strcpy(search, "Most Responded");
					}

					if (strlen(search)) {
						sprintf(str, "eTube Standard Search: <em><b>%s</></> videos", search);
					} else {
						sprintf(str, "eTube Search");
					}
				} else {
					sprintf(str, "eTube Search: <em><b>%s</></>", et->search->query);
				}
				
				et->search->query = elm_entry_utf8_to_markup(smartNewLine(str, 30));
				htmlCharFix(et->search->query); // It's needed - bad hack. To fix
				elm_entry_entry_set(et->etwin->frametitle, et->search->query);
			}
			
			//free(et->search->query); // to free when got the results!
		}
		//free(et->search);
	}
}

static void etube_fav_search(EtubeStruct *et, char *fav) {
	// TODO All this must be put in a search function that also prevents contemporary searches
	et->search = calloc(1, sizeof(EtubeSearch));
	et->search->query = malloc(strlen(fav)+1);
	strcpy(et->search->query, fav);
	et->search->standard = true;
	
	elm_entry_entry_set(et->etwin->inwintext, "<em>Searching Videos...</>");
	evas_object_show(et->etwin->inwin);
	etubecon_init(et);
	etube_videolist_free(et);
	etube_search_feed(et->etcon, fav, "this_week");
	etube_frame_title_set(et);
	ecore_timer_add(0.5, etube_videolist_check, et);
}

static void etube_top_rated_search_cb(void *data, Evas_Object *obj, void *event_info) {
	EtubeStruct *et;
	et = data;
	
	etube_fav_search(et, "top_rated");
}

static void etube_top_fav_search_cb(void *data, Evas_Object *obj, void *event_info) {
	EtubeStruct *et;
	et = data;
	
	etube_fav_search(et, "top_favorites");
}

static void etube_most_view_search_cb(void *data, Evas_Object *obj, void *event_info) {
	EtubeStruct *et;
	et = data;
	
	etube_fav_search(et, "top_rated");
}

static void etube_most_pop_search_cb(void *data, Evas_Object *obj, void *event_info) {
	EtubeStruct *et;
	et = data;
	
	etube_fav_search(et, "most_popular");
}

static void etube_most_disc_search_cb(void *data, Evas_Object *obj, void *event_info) {
	EtubeStruct *et;
	et = data;
	
	etube_fav_search(et, "most_discussed");
}

static void etube_most_link_search_cb(void *data, Evas_Object *obj, void *event_info) {
	EtubeStruct *et;
	et = data;
	
	etube_fav_search(et, "most_linked");
}

static void etube_searchbar_hide(void *data, Evas_Object *obj, void *event_info) {
	EtubeWindow *etwin;
	etwin = data;
	
	if (etwin->searchscroll) {
		evas_object_del(etwin->searchscroll);
		etwin->searchscroll = NULL;
	}
	
	if (etwin->searchentry) {
		evas_object_del(etwin->searchentry);
		etwin->searchentry = NULL;
	}

	evas_object_hide(etwin->searchbox);
}

static void etube_buttons_del(EtubeWindow *etwin) {

	if (etwin->lbutton)
		evas_object_del(etwin->lbutton);
	
	if (etwin->lbutton)
		evas_object_del(etwin->rbutton);
}

static void etube_search_mode_cb(void *data, Evas_Object *obj, void *event_info);

static void etube_stdbuttons_show(void *data, Evas_Object *obj, void *event_info) {
	Evas_Object *bt;
	EtubeStruct *et;
	et = data;

	etube_searchbar_hide(et->etwin, NULL, NULL);
	etube_buttons_del(et->etwin);

	et->etwin->lbutton = elm_button_add(et->etwin->window);
	bt = et->etwin->lbutton;
	elm_button_label_set(bt, "Search");
	evas_object_smart_callback_add(bt, "clicked", etube_search_mode_cb, et);
	evas_object_size_hint_align_set(bt, -1.0, -1.0);
	evas_object_size_hint_weight_set(bt, 1.0, 0.0);
	elm_box_pack_end(et->etwin->buttonsbox, bt);
	evas_object_show(bt);
	
	et->etwin->rbutton = elm_hoversel_add(et->etwin->window);
	bt = et->etwin->rbutton;
	elm_hoversel_hover_parent_set(bt, et->etwin->window);
	elm_hoversel_label_set(bt, "   Weekly Top   "); // workaround
	elm_hoversel_item_add(bt, "Top Rated", NULL, ELM_ICON_NONE, etube_top_rated_search_cb, et);
	elm_hoversel_item_add(bt, "Top Favorites", NULL, ELM_ICON_NONE, etube_top_fav_search_cb, et);
	elm_hoversel_item_add(bt, "Most Viewed", NULL, ELM_ICON_NONE, etube_most_view_search_cb, et);
	elm_hoversel_item_add(bt, "Most Popualar", NULL, ELM_ICON_NONE, etube_most_pop_search_cb, et);
	elm_hoversel_item_add(bt, "Most Discussed", NULL, ELM_ICON_NONE, etube_most_disc_search_cb, et);
	elm_hoversel_item_add(bt, "Most Linked", NULL, ELM_ICON_NONE, etube_most_link_search_cb, et);
	evas_object_smart_callback_add(bt, "clicked", etube_searchbar_hide, et->etwin);
	evas_object_size_hint_weight_set(bt, 0.0, 0.0);
	evas_object_size_hint_align_set(bt, 0.5, 0.5);
	elm_box_pack_end(et->etwin->buttonsbox, bt);
	evas_object_show(bt);
}

static void etube_str_search_cb(void *data, Evas_Object *obj, void *event_info) {
	EtubeStruct *et;
	size_t len;
	
	et = data;
	len = strlen(elm_entry_entry_get(et->etwin->searchentry)) - 4;
	
	if (len) {
		et->search = calloc(1, sizeof(EtubeSearch));
		et->search->standard = false;
		et->search->query = malloc(len+1);
		strncpy(et->search->query, elm_entry_entry_get(et->etwin->searchentry), len);
		et->search->query[len] = '\0';
		trimString(et->search->query);

		// TODO All this must be put in a search function that also prevents contemporary searches
		char *str;
		asprintf(&str, "<em>Searching '%s' Videos...</>",
					elm_entry_utf8_to_markup(smartNewLine(et->search->query, 25)));
		elm_entry_entry_set(et->etwin->inwintext, str);
		free(str);
		evas_object_show(et->etwin->inwin);
		etubecon_init(et);
		etube_videolist_free(et);
		etube_search_videos(et->etcon, urlEscape(et->search->query));
		etube_frame_title_set(et);
		ecore_timer_add(0.5, etube_videolist_check, et);
	}
	
	etube_stdbuttons_show(et, NULL, NULL);
}

static void etube_searchbuttons_show(EtubeStruct *et) {
	Evas_Object *bt;

	etube_buttons_del(et->etwin);

	et->etwin->lbutton = elm_button_add(et->etwin->window);
	bt = et->etwin->lbutton;
	elm_button_label_set(bt, "Cancel");
	evas_object_size_hint_align_set(bt, -1.0, -1.0);
	evas_object_size_hint_weight_set(bt, 1.0, 0.0);
	elm_box_pack_end(et->etwin->buttonsbox, bt);
	evas_object_show(bt);
	evas_object_smart_callback_add(bt, "clicked", etube_stdbuttons_show, et);

	et->etwin->rbutton = elm_button_add(et->etwin->window);
	bt =et->etwin->rbutton;
	elm_button_label_set(bt, "Go!");
	evas_object_size_hint_align_set(bt, -1.0, -1.0);
	evas_object_size_hint_weight_set(bt, 1.0, 0.0);
	elm_box_pack_end(et->etwin->buttonsbox, bt);
	evas_object_show(bt);
	evas_object_smart_callback_add(bt, "clicked", etube_str_search_cb, et);
}

static void etube_search_mode_cb(void *data, Evas_Object *obj, void *event_info) {
	Evas_Object *sc, *en;
	EtubeStruct *et;
	et = data;

	if (!et->etwin->searchscroll) {
		et->etwin->searchscroll = sc = elm_scroller_add(et->etwin->window);
		elm_scroller_content_min_limit(sc, 0, 1);
		evas_object_size_hint_weight_set(sc, 1.0, 0.0);
		evas_object_size_hint_align_set(sc, -1.0, -1.0);
		elm_box_pack_end(et->etwin->searchbox, sc);
		evas_object_show(sc);
	}
	
	if (!et->etwin->searchentry) {
		et->etwin->searchentry = en = elm_entry_add(et->etwin->window);;
		elm_entry_single_line_set(en, true);
		elm_entry_editable_set(en, true);
		elm_entry_line_wrap_set(en, true);
		evas_object_size_hint_weight_set(en, 1.0, 0.0);
		evas_object_size_hint_align_set(en, -1.0, 0.0);
		elm_scroller_content_set(sc, en);
		evas_object_show(en);
		
		evas_object_smart_callback_add(et->etwin->searchentry, "activated", etube_str_search_cb, et);
	}
	
	evas_object_show(et->etwin->searchbox);
	etube_searchbuttons_show(et);
}

void etube_videos_window(EtubeStruct *et) {
	Evas_Object *win, *bg, *ic, *li, *bx, *bx2, *bt, *lb, *sc, *fr;
	
	printf("DEBUG: <elm_main>\n");
	
	et->etwin->window = elm_win_add(NULL, "eTube", ELM_WIN_BASIC);
	win = et->etwin->window;
	elm_win_title_set(win, "eTube "ETUBE_VERSION);
	
	evas_object_smart_callback_add(win, "delete-request", win_del, NULL);
	elm_win_autodel_set(win, 1);
	
	bg = elm_bg_add(win);
	elm_win_resize_object_add(win, bg);
	evas_object_size_hint_weight_set(bg, 1.0, 1.0);
	evas_object_show(bg);
	printf("DEBUG: bg created\n");
	
	bx = elm_box_add(win);
	evas_object_size_hint_weight_set(bx, 1.0, 1.0);
	elm_win_resize_object_add(win, bx);
	evas_object_show(bx);
	printf("DEBUG: bx created\n");
	
	bx2 = elm_box_add(win);
	elm_box_horizontal_set(bx2, true);
	elm_box_homogenous_set(bx2, false);
	evas_object_size_hint_weight_set(bx2, 1.0, 0.0);
	evas_object_size_hint_align_set(bx2, 0.5, 0.5);
	
	fr = elm_frame_add(win);
	elm_frame_style_set(fr, "outdent_top");
	evas_object_size_hint_weight_set(fr, 0.0, 0.0);
	evas_object_size_hint_align_set(fr, 0.5, 0.5);
	elm_box_pack_end(bx, fr);
	evas_object_show(fr);

	et->etwin->frametitle = elm_entry_add(win);
	elm_entry_single_line_set(et->etwin->frametitle, FALSE);
	elm_entry_editable_set(et->etwin->frametitle, FALSE);
	elm_entry_line_wrap_set(et->etwin->frametitle, FALSE);
	elm_entry_entry_set(et->etwin->frametitle, "eTube Videos");
	evas_object_size_hint_weight_set(et->etwin->frametitle, 1.0, 0.0);
	evas_object_size_hint_align_set(et->etwin->frametitle, 0.5, 0.5);
	elm_frame_content_set(fr, et->etwin->frametitle);
	evas_object_show(et->etwin->frametitle);
	
	et->etwin->scrollist = elm_list_add(win);
	li = et->etwin->scrollist;
	evas_object_size_hint_align_set(li, -1.0, -1.0);
	evas_object_size_hint_weight_set(li, 1.0, 1.0);
	elm_list_horizontal_mode_set(li, ELM_LIST_COMPRESS);
	elm_box_pack_end(bx, li);
	evas_object_show(li);
	elm_list_go(li);
	
	et->etwin->searchbox = elm_box_add(win);
	bx2 = et->etwin->searchbox;
	elm_box_horizontal_set(bx2, 1);
	elm_box_homogenous_set(bx2, 1);
	evas_object_size_hint_weight_set(bx2, 1.0, 0.0);
	evas_object_size_hint_align_set(bx2, -1.0, -1.0);
	elm_box_pack_end(bx, bx2);
	evas_object_show(bx2);
	
	et->etwin->buttonsbox = elm_box_add(win);
	bx2 = et->etwin->buttonsbox;
	elm_box_horizontal_set(bx2, 1);
	elm_box_homogenous_set(bx2, 1);
	evas_object_size_hint_weight_set(bx2, 1.0, 0.0);
	evas_object_size_hint_align_set(bx2, -1.0, -1.0);
	elm_box_pack_end(bx, bx2);
	evas_object_show(bx2);
	
	etube_stdbuttons_show(et, NULL, NULL) ;
	
	et->etwin->etubepager = elm_pager_add(win);
	elm_pager_content_push(et->etwin->etubepager, bx);
	elm_win_resize_object_add(win, et->etwin->etubepager);
	evas_object_show(et->etwin->etubepager);
	et->etwin->videospage = bx;
	
	et->etwin->inwin = elm_win_inwin_add(win);
	elm_win_inwin_style_set(et->etwin->inwin, "minimal_vertical");
	evas_object_show(et->etwin->inwin);

	bx2 = elm_box_add(win);
	elm_box_horizontal_set(bx2, false);
	elm_box_homogenous_set(bx2, true);
	evas_object_size_hint_weight_set(bx2, 1.0, 0.0);
	evas_object_size_hint_align_set(bx2, 0.5, 0.5);
	elm_win_inwin_content_set(et->etwin->inwin, bx2);
	
	et->etwin->inwintext = elm_entry_add(win);
	elm_entry_single_line_set(et->etwin->inwintext, FALSE);
	elm_entry_editable_set(et->etwin->inwintext, FALSE);
	elm_entry_line_wrap_set(et->etwin->inwintext, FALSE);
	elm_entry_entry_set(et->etwin->inwintext, "<em>Loading Videos...</>");
	elm_box_pack_end(bx2, et->etwin->inwintext);
	evas_object_show(et->etwin->inwintext);
	
// 	bt = elm_button_add(win);
// 	elm_button_label_set(bt, "Cancel");
// 	evas_object_smart_callback_add(bt, "clicked", NULL, NULL);
// 	evas_object_size_hint_weight_set(bt, 1.0, 0.0);
// 	elm_box_pack_end(bx2, bt);
// 	evas_object_show(bt);

	evas_object_resize(win, 480, 640);

	printf("DEBUG: showing window .. ");
	evas_object_show(win);
	printf("showed!\n");
}

void etube_video_fetch(EtubeStruct *et) {
// 	eina_list_init();
	etubecon_init(et);
	etube_videolist_free(et);
	
	et->search = calloc(1, sizeof(EtubeSearch));
	et->search->query = malloc(strlen("openmoko")+1);
	strcpy(et->search->query, "openmoko");
	et->search->standard = false;
	
	etube_search_videos(et->etcon, et->search->query);
	ecore_timer_add(0.5, etube_videolist_check, et);
}

void etube_init(EtubeStruct *et) {
	et->etcon = NULL;
	et->videolist = NULL;
	et->etwin = calloc(1, sizeof(EtubeWindow));
	ecore_file_mkdir(TMP_PATH);
}

void etube_shutdown(EtubeStruct *et) {
	etubecon_shutdown(et);
	etube_videolist_free(et);
	free(et->etcon);
	free(et->etwin);
	ecore_file_recursive_rm(TMP_PATH);
}

EAPI int elm_main(int argc, char ** argv) {
	EtubeStruct et;
	
	etube_init(&et);
	etube_video_fetch(&et);
	etube_videos_window(&et);
	
	elm_run();
	
	elm_shutdown();
	
	etube_shutdown(&et);
	
	printf("DEBUG: </elm_main>\n");
}

ELM_MAIN()
