// To be licensed -- A telefoninux.org-community stub.

// gcc `pkg-config --cflags --libs elementary` gui-main.c  -o coso

#include <Elementary.h>
#include <Ecore.h>
#include <Ecore_Con.h>

typedef unsigned char bool;
#define false 0
#define true 1

typedef struct _EtubeConData EtubeConData;
struct _EtubeConData {
    char *data;
    int data_size;
};

typedef struct _EtubeHttpConRequest EtubeHttpConRequest;
struct _EtubeHttpConRequest {
    char *host;
    char *type;
    char *headers;
    char *target;
    char *content;
};

typedef struct _EtubeHttpConReply EtubeHttpConReply;
struct _EtubeHttpConReply {
    char *data;
    unsigned int code;
};

typedef struct _EtubeCon EtubeCon;
struct _EtubeCon {
	Ecore_Con_Server *server;
	Ecore_Event_Handler *connection;
	Ecore_Event_Handler *transfer;
	Ecore_Event_Handler *disconnect;
	EtubeHttpConRequest *request;
	EtubeHttpConReply *reply;
	EtubeConData *condata;
};

typedef struct _YTVideo YTVideo;
struct _YTVideo {
	char *id;
	char *title;
	char *author;
	char *thumbnail;
	char *description;
	unsigned int duration;
	unsigned int rating; // data not grabbed yet
	unsigned int raters; // data not grabbed yet
};

typedef struct _Testitem
{
   Elm_Genlist_Item *item;
   int mode;
   int onoff;
} Testitem;

// this fun adds a edje layout,
// to be used asap
void window_swallow(Evas_Object * ly, const char *key, Evas_Object *object) {
  if(ly!=NULL)
    edje_object_part_swallow(elm_layout_edje_get(ly),key,object);
}

// close win and free every elm part
static void win_del(void * data, Evas_Object *obj, void *event_info) {

  printf("DEBUG: <win_del>\n");
  elm_exit();
  printf("DEBUG: </win_del>\n");


}

void gl3_del(const void *data, Evas_Object *obj) {
  printf("DEBUG: <gl3_del />\n");

}

Evas_Bool gl3_state_get(const void *data, Evas_Object *obj, const char *part) {
   printf("DEBUG: <gl3_state_get />\n");
   return 0;
}

// action to do when we select an item from list
gl_sel(void *data, Evas_Object *obj, void *event_info) {
   printf("sel item data [%p] on genlist obj [%p], item pointer [%p]\n", data, obj, event_info);
}

// button search callback, let's search for videos
static void my_gl_update(void *data, Evas_Object *obj, void *event_info) {
   printf("DEBUG: <my_gl_update>\n");
   Testitem *tit = data;
   tit->mode++;
   elm_genlist_item_update(tit->item);
   printf("DEBUG: </my_gl_update>\n");
}

static Elm_Genlist_Item_Class itc3;
char *gl3_label_get(const void *data, Evas_Object *obj, const char *part) {

   printf("DEBUG: <gl3_label_get>\n");
   Testitem *tit = (Testitem *) data;
   char buf[256];
   snprintf(buf, sizeof(buf), "Item mode %i", tit->mode);
   printf("DEBUG: </gl3_label_get>\n");
   return strdup(buf);
}

// when we select the checkbox
static void my_gl_item_check_changed(void *data, Evas_Object *obj, void *event_info) {

   printf("DEBUG: <my_gl_item_check_changed>\n");
   Testitem *tit = data;
   tit->onoff = elm_check_state_get(obj);
   printf("item %p onoff = %i\n", tit, tit->onoff);
   printf("DEBUG: </my_gl_item_check_changed>\n");
}


Evas_Object *gl3_icon_get(const void *data, Evas_Object *obj, const char *part) {

   Testitem *tit = (Testitem *) data;
   char buf[PATH_MAX];

   printf("DEBUG: <gl3_icon_get>\n");
   if (!strcmp(part, "elm.swallow.icon")) {

        Evas_Object *bx = elm_box_add(obj);
        Evas_Object *ic;
        elm_box_horizontal_set(bx, 1);
        ic = elm_icon_add(obj);
        snprintf(buf, sizeof(buf), "%s/images/logo_small.png", "/usr/share/elementary/");
        elm_icon_file_set(ic, buf, NULL);
        elm_icon_scale_set(ic, 0, 0);
        evas_object_show(ic);
        elm_box_pack_end(bx, ic);
        ic = elm_icon_add(obj);
        elm_icon_file_set(ic, buf, NULL);
        elm_icon_scale_set(ic, 0, 0);
        evas_object_show(ic);
        elm_box_pack_end(bx, ic);
        evas_object_show(bx);
	printf("DEBUG: return bx\n");
        printf("DEBUG: </gl3_icon_get>\n");
        return bx;

   } else if (!strcmp(part, "elm.swallow.end")) {

        Evas_Object *ck;
        ck = elm_check_add(obj);
        elm_check_state_set(ck, tit->onoff);
        evas_object_smart_callback_add(ck, "changed", my_gl_item_check_changed, data);
        evas_object_show(ck);
	printf("DEBUG: return ck\n");
        printf("DEBUG: </gl3_icon_get>\n");
        return ck;

   }

   printf("DEBUG: </gl3_icon_get>\n");
   return NULL;
}

EAPI int elm_main(int argc, char ** argv) {

   // first test the downside
//   downside_main(0, "");

   Evas_Object *win, *bg, *gl, *bx, *bx2, *bt;
   static Testitem tit[3];
   static YTVideo ytv[3];
   int i;
   
   printf("DEBUG: <elm_main>\n");

   win = elm_win_add(NULL, "eTube", ELM_WIN_BASIC);
   elm_win_title_set(win, "eTube 0.0.1a");

   evas_object_smart_callback_add(win, "delete-request", win_del, NULL);
   elm_win_autodel_set(win, 1);

   bg = elm_bg_add(win);
   elm_win_resize_object_add(win, bg);
   evas_object_size_hint_weight_set(bg, 1.0, 1.0);
   evas_object_show(bg);
   printf("DEBUG: bg created\n");
   
   bx = elm_box_add(win);
   evas_object_size_hint_weight_set(bx, 1.0, 1.0);
   elm_win_resize_object_add(win, bx);
   evas_object_show(bx);
   printf("DEBUG: bx created\n");

   gl = elm_genlist_add(win);
   evas_object_size_hint_align_set(gl, -1.0, -1.0);
   evas_object_size_hint_weight_set(gl, 1.0, 1.0);
   evas_object_show(gl);
   printf("DEBUG: gl created\n");

   itc3.item_style     = "default";
   itc3.func.label_get = gl3_label_get;
   itc3.func.icon_get  = gl3_icon_get;
   itc3.func.state_get = gl3_state_get;
   itc3.func.del       = gl3_del;

   Eina_List *list = NULL;
   Eina_List *l;
   const char *file;

   // here we have to get data from downside
/*   list = eina_list_append(list, "dsc01.jpg");
   list = eina_list_append(list, "blah");
   list = eina_list_append(list, "dsc10.jpg");
*/
   // get data from downside
   printf("DEBUG: econ\n");
   EtubeCon econ;
   printf("DEBUG: etubecon_init\n");
   etubecon_init(&econ);

   ecore_init();
   ecore_con_init();
   eina_list_init();

   printf("DEBUG: etube_search_videos\n");
   etube_search_videos(&econ, NULL); // FIXME: segfault if not etube_con_init

   //printf("DEBUG: ecore_main_loop_begin\n");
   //ecore_main_loop_begin();	// we need it with parseRSSVideo ?

//   printf("DEBUG: parseRSSVideoList\n");
//   parseRSSVideoList(econ.reply->data, &list); // FIXME: segfault without connection

   if (list != NULL) {
      printf("DEBUG: <EINA_LIST_FOREACH>\n");
      /* print the list */
      for (l = list, file = eina_list_data_get(l); l; l = eina_list_next(l), file = eina_list_data_get(l)) {
         printf("DEBUG: eina_list_data: ");
	 printf("%s\n", file);

	 // tit[0].mode is an int, file is a string
	 //      sprintf((char *)tit[0].mode, file) ;
	 //      tit[0].mode = 0;
	 //      tit[0].item = elm_genlist_item_append(gl, &itc3, &(tit[0])/* item data */, NULL/* parent */, ELM_GENLIST_ITEM_NONE, (void *) gl_sel/* func */, NULL/* func data */);
      }
      printf("DEBUG: </EINA_LIST_FOREACH>\n");
   } else {
      // no results from yt, let's show something else
      printf("DEBUG: list is NULL\n");
   }

   // EAPI Elm_Genlist_Item *elm_genlist_item_append(Evas_Object *obj, const Elm_Genlist_Item_Class *itc, const void *data, Elm_Genlist_Item *parent, Elm_Genlist_Item_Flags flags, void (*func) (void *data, Evas_Object *obj, void *event_info), const void *func_data);
   tit[0].mode = 0;
   tit[0].item = elm_genlist_item_append(gl, &itc3, &(tit[0])/* item data */, NULL/* parent */, ELM_GENLIST_ITEM_NONE, (void *) gl_sel/* func */, NULL/* func data */);
   tit[1].mode = 1;
   tit[1].item = elm_genlist_item_append(gl, &itc3, &(tit[1])/* item data */, NULL/* parent */, ELM_GENLIST_ITEM_NONE, (void *) gl_sel/* func */, NULL/* func data */);
   tit[2].mode = 2;
   tit[2].item = elm_genlist_item_append(gl, &itc3, &(tit[2])/* item data */, NULL/* parent */, ELM_GENLIST_ITEM_NONE, (void *) gl_sel/* func */, NULL/* func data */);
   
   printf("DEBUG: <elm_box_pack_end> .");
   elm_box_pack_end(bx, gl);
   printf(". </elm_box_pack_end>\n");

   printf("DEBUG: <evas_object_show> .");
   evas_object_show(bx2);
   printf(". </evas_object_show>\n");
   
   bx2 = elm_box_add(win);
   elm_box_horizontal_set(bx2, 1);
   elm_box_homogenous_set(bx2, 1);
   evas_object_size_hint_weight_set(bx2, 1.0, 0.0);
   evas_object_size_hint_align_set(bx2, -1.0, -1.0);
   printf("DEBUG: bx2 created\n");
   
   bt = elm_button_add(win);
   elm_button_label_set(bt, "search");
   evas_object_smart_callback_add(bt, "clicked", my_gl_update, &(tit[0]));
   evas_object_size_hint_align_set(bt, -1.0, -1.0);
   evas_object_size_hint_weight_set(bt, 1.0, 0.0);
   elm_box_pack_end(bx2, bt);
   evas_object_show(bt);
   printf("DEBUG: bt created\n");
  
  /* 
   bt = elm_button_add(win);
   elm_button_label_set(bt, "[2]");
   evas_object_smart_callback_add(bt, "clicked", my_gl_update, &(tit[1]));
   evas_object_size_hint_align_set(bt, -1.0, -1.0);
   evas_object_size_hint_weight_set(bt, 1.0, 0.0);
   elm_box_pack_end(bx2, bt);
   evas_object_show(bt);
  */

   Evas_Object *en;
   en = elm_entry_add(win);
   elm_entry_line_wrap_set(en, 0);
   elm_entry_entry_set(en, "");
   evas_object_size_hint_weight_set(en, 1.0, 1.0);
   evas_object_size_hint_align_set(en, -1.0, -1.0);
   elm_entry_single_line_set(en, 1);
   elm_box_pack_end(bx2, en);
   evas_object_show(en);
   printf("DEBUG: en created\n");


  /* 
   bt = elm_button_add(win);
   elm_button_label_set(bt, "[3]");
   evas_object_smart_callback_add(bt, "clicked", my_gl_update, &(tit[2]));
   evas_object_size_hint_align_set(bt, -1.0, -1.0);
   evas_object_size_hint_weight_set(bt, 1.0, 0.0);
   elm_box_pack_end(bx2, bt);
   evas_object_show(bt);
   */
   elm_box_pack_end(bx, bx2);
   evas_object_show(bx2);
   printf("DEBUG: bx2 showed\n");

   evas_object_resize(win, 320, 320);
   printf("DEBUG: showing window .. ");
   evas_object_show(win);
   printf("showed!\n");

   elm_run();
   elm_shutdown();

   printf("DEBUG: </elm_main>\n");

}

ELM_MAIN()

